@@ -1,9 +1,13 @@
 -- global module
--- validates certificates for all hosts used for s2soutinjection
+-- validates certificates for all hosts used for s2soutinjection or s2sout_override
 module:set_global();
 
 local s2s_overrides = module:get_option("s2s_connect_overrides");
 
+if not s2s_overrides then
+    s2s_overrides = module:get_option("s2sout_override");
+end
+
 function attach(event)
     local session = event.session;
 
